@@ -1,7 +1,7 @@
 package Unix::Passwd::File;
 
 our $DATE = '2015-03-06'; # DATE
-our $VERSION = '0.14'; # VERSION
+our $VERSION = '0.15'; # VERSION
 
 use 5.010001;
 use strict;
@@ -235,6 +235,13 @@ for (keys %gshadow_fields) {
     delete $gshadow_fields{$_}{pos};
 }
 
+sub _arg_from_field {
+    my ($fields, $name, %extra) = @_;
+    my %spec = %{ $fields->{$name} };
+    $spec{$_} = $extra{$_} for keys %extra;
+    ($name => \%spec);
+}
+
 sub _backup {
     my ($fh, $path) = @_;
     seek $fh, 0, 0 or return [500, "Can't seek: $!"];
@@ -561,6 +568,9 @@ Either `user` OR `uid` must be specified.
 The function is not dissimilar to Unix's `getpwnam()` or `getpwuid()`.
 
 _
+    args_groups => [
+        {rel=>'one_of', args=>[qw/user uid/]},
+    ],
     args => {
         %common_args,
         user => {
@@ -620,6 +630,9 @@ sub get_user {
 $SPEC{user_exists} = {
     v => 1.1,
     summary => 'Check whether user exists',
+    args_groups => [
+        {rel=>'one_of', args=>[qw/user uid/]},
+    ],
     args => {
         %common_args,
         user => {
@@ -712,6 +725,9 @@ Either `group` OR `gid` must be specified.
 The function is not dissimilar to Unix's `getgrnam()` or `getgrgid()`.
 
 _
+    args_groups => [
+        {rel=>'one_of', args=>[qw/group gid/]},
+    ],
     args => {
         %common_args,
         group => {
@@ -842,6 +858,9 @@ sub list_users_and_groups {
 $SPEC{group_exists} = {
     v => 1.1,
     summary => 'Check whether group exists',
+    args_groups => [
+        {rel=>'one_of', args=>[qw/group gid/]},
+    ],
     args => {
         %common_args,
         group => {
@@ -870,7 +889,9 @@ $SPEC{get_user_groups} = {
     args => {
         %common_args,
         user => {
+            schema => 'str*',
             req => 1,
+            pos => 0,
         },
         detail => {
             summary => 'If true, return all fields instead of just group names',
@@ -942,10 +963,14 @@ $SPEC{is_member} = {
     args => {
         %common_args,
         user => {
+            schema => 'str*',
             req => 1,
+            pos => 0,
         },
         group => {
+            schema => 'str*',
             req => 1,
+            pos => 0,
         },
     },
     result_naked => 1,
@@ -1173,7 +1198,9 @@ $SPEC{add_group} = {
         %common_args,
         %write_args,
         group => {
+            schema => 'str*',
             req => 1,
+            pos => 0,
         },
         gid => {
             summary => 'Pick a specific new GID',
@@ -1219,7 +1246,9 @@ $SPEC{add_user} = {
         %common_args,
         %write_args,
         user => {
+            schema => 'str*',
             req => 1,
+            pos => 0,
         },
         group => {
             summary => 'Select primary group '.
@@ -1444,9 +1473,13 @@ _
     args => {
         %common_args,
         %write_args,
-        map( {($_=>$group_fields{$_})} qw/group pass gid members/),
-        map( {($_=>$gshadow_fields{$_})}
-                 qw/encpass admins/),
+        _arg_from_field(\%group_fields, 'group', req=>1, pos=>0),
+        _arg_from_field(\%group_fields, 'pass'),
+        _arg_from_field(\%group_fields, 'gid'),
+        _arg_from_field(\%group_fields, 'members'),
+
+        _arg_from_field(\%gshadow_fields, 'encpass'),
+        _arg_from_field(\%gshadow_fields, 'admins'),
     },
 };
 sub modify_group {
@@ -1465,10 +1498,20 @@ _
     args => {
         %common_args,
         %write_args,
-        map( {($_=>$passwd_fields{$_})} qw/user pass uid gid gecos home shell/),
-        map( {($_=>$shadow_fields{$_})}
-                 qw/encpass last_pwchange min_pass_age max_pass_age
-                   pass_warn_period pass_inactive_period expire_date/),
+        _arg_from_field(\%passwd_fields, 'user', req=>1, pos=>0),
+        _arg_from_field(\%passwd_fields, 'uid'),
+        _arg_from_field(\%passwd_fields, 'gid'),
+        _arg_from_field(\%passwd_fields, 'gecos'),
+        _arg_from_field(\%passwd_fields, 'home'),
+        _arg_from_field(\%passwd_fields, 'shell'),
+
+        _arg_from_field(\%shadow_fields, 'encpass'),
+        _arg_from_field(\%shadow_fields, 'last_pwchange'),
+        _arg_from_field(\%shadow_fields, 'min_pass_age'),
+        _arg_from_field(\%shadow_fields, 'max_pass_age'),
+        _arg_from_field(\%shadow_fields, 'pass_warn_period'),
+        _arg_from_field(\%shadow_fields, 'pass_inactive_period'),
+        _arg_from_field(\%shadow_fields, 'expire_date'),
     },
 };
 sub modify_user {
@@ -1481,10 +1524,14 @@ $SPEC{add_user_to_group} = {
     args => {
         %common_args,
         user => {
+            schema => 'str*',
             req => 1,
+            pos => 0,
         },
         group => {
+            schema => 'str*',
             req => 1,
+            pos => 1,
         },
     },
 };
@@ -1516,10 +1563,14 @@ $SPEC{delete_user_from_group} = {
     args => {
         %common_args,
         user => {
+            schema => 'str*',
             req => 1,
+            pos => 0,
         },
         group => {
+            schema => 'str*',
             req => 1,
+            pos => 1,
         },
     },
 };
@@ -1567,7 +1618,9 @@ _
     args => {
         %common_args,
         user => {
+            schema => 'str*',
             req => 1,
+            pos => 0,
         },
         add_to => {
             summary => 'List of group names to add the user as member of',
@@ -1625,12 +1678,16 @@ $SPEC{set_user_groups} = {
     args => {
         %common_args,
         user => {
+            schema => 'str*',
             req => 1,
+            pos => 0,
         },
         groups => {
             summary => 'List of group names that user is member of',
             schema => [array => {of=>'str*', default=>[]}],
             req => 1,
+            pos => 1,
+            greedy => 1,
             description => <<'_',
 
 Aside from this list, user will not belong to any other group.
@@ -1685,10 +1742,14 @@ $SPEC{set_user_password} = {
         %common_args,
         %write_args,
         user => {
+            schema => 'str*',
             req => 1,
+            pos => 0,
         },
         pass => {
+            schema => 'str*',
             req => 1,
+            pos => 1,
         },
     },
 };
@@ -1801,7 +1862,9 @@ $SPEC{delete_group} = {
         %common_args,
         %write_args,
         group => {
+            schema => 'str*',
             req => 1,
+            pos => 0,
         },
     },
 };
@@ -1816,7 +1879,9 @@ $SPEC{delete_user} = {
         %common_args,
         %write_args,
         user => {
+            schema => 'str*',
             req => 1,
+            pos => 0,
         },
     },
 };
@@ -1839,7 +1904,7 @@ Unix::Passwd::File - Manipulate /etc/{passwd,shadow,group,gshadow} entries
 
 =head1 VERSION
 
-This document describes version 0.14 of Unix::Passwd::File (from Perl distribution Unix-Passwd-File), released on 2015-03-06.
+This document describes version 0.15 of Unix::Passwd::File (from Perl distribution Unix-Passwd-File), released on 2015-03-06.
 
 =head1 SYNOPSIS
 
@@ -1946,7 +2011,7 @@ List of group names to remove the user as member of.
 
 Specify location of passwd files.
 
-=item * B<user>* => I<any>
+=item * B<user>* => I<str>
 
 =back
 
@@ -1987,7 +2052,7 @@ Pick a specific new GID.
 
 Adding a new group with duplicate GID is allowed.
 
-=item * B<group>* => I<any>
+=item * B<group>* => I<str>
 
 =item * B<max_gid> => I<int> (default: 65535)
 
@@ -2127,7 +2192,7 @@ Pick a specific new UID.
 
 Adding a new user with duplicate UID is allowed.
 
-=item * B<user>* => I<any>
+=item * B<user>* => I<str>
 
 =back
 
@@ -2155,9 +2220,9 @@ Arguments ('*' denotes required arguments):
 
 Specify location of passwd files.
 
-=item * B<group>* => I<any>
+=item * B<group>* => I<str>
 
-=item * B<user>* => I<any>
+=item * B<user>* => I<str>
 
 =back
 
@@ -2192,7 +2257,7 @@ will not be backed up. Previous backup will be overwritten.
 
 Specify location of passwd files.
 
-=item * B<group>* => I<any>
+=item * B<group>* => I<str>
 
 =back
 
@@ -2227,7 +2292,7 @@ will not be backed up. Previous backup will be overwritten.
 
 Specify location of passwd files.
 
-=item * B<user>* => I<any>
+=item * B<user>* => I<str>
 
 =back
 
@@ -2255,9 +2320,9 @@ Arguments ('*' denotes required arguments):
 
 Specify location of passwd files.
 
-=item * B<group>* => I<any>
+=item * B<group>* => I<str>
 
-=item * B<user>* => I<any>
+=item * B<user>* => I<str>
 
 =back
 
@@ -2425,7 +2490,7 @@ If true, return all fields instead of just group names.
 
 Specify location of passwd files.
 
-=item * B<user>* => I<any>
+=item * B<user>* => I<str>
 
 =item * B<with_field_names> => I<bool> (default: 1)
 
@@ -2483,9 +2548,9 @@ Arguments ('*' denotes required arguments):
 
 Specify location of passwd files.
 
-=item * B<group>* => I<any>
+=item * B<group>* => I<str>
 
-=item * B<user>* => I<any>
+=item * B<user>* => I<str>
 
 =back
 
@@ -2642,7 +2707,7 @@ Specify location of passwd files.
 
 Numeric group ID.
 
-=item * B<group> => I<str>
+=item * B<group>* => I<str>
 
 Group name.
 
@@ -2722,10 +2787,6 @@ The number of days after which the user will have to change her password.
 
 The number of days the user will have to wait before she will be allowed to change her password again.
 
-=item * B<pass> => I<str>
-
-Password, generally should be "x" which means password is encrypted in shadow.
-
 =item * B<pass_inactive_period> => I<int>
 
 The number of days after a password has expired (see max_pass_age) during which the password should still be accepted (and user should update her password during the next login).
@@ -2742,7 +2803,7 @@ User's home directory.
 
 Numeric user ID.
 
-=item * B<user> => I<str>
+=item * B<user>* => I<str>
 
 User (login) name.
 
@@ -2778,7 +2839,7 @@ List of group names that user is member of.
 
 Aside from this list, user will not belong to any other group.
 
-=item * B<user>* => I<any>
+=item * B<user>* => I<str>
 
 =back
 
@@ -2813,9 +2874,9 @@ will not be backed up. Previous backup will be overwritten.
 
 Specify location of passwd files.
 
-=item * B<pass>* => I<any>
+=item * B<pass>* => I<str>
 
-=item * B<user>* => I<any>
+=item * B<user>* => I<str>
 
 =back
 
